import java.util.*;
import java.text.*;
/**
 * Clase que representa una Factura del Sistema de Facturacion
 * 
 * @author G. Cherencio
 * @version 1.0
 */
public class Factura {
    // ATRIBUTOS
    private int nro;
    private Date fecha;
    private Cliente cliente     = null;
    private boolean confirmada  = false;
    private double monto        = 0.0;
    private boolean pagada      = false;
    private Detalle detalle[]   = new Detalle[20];
    private int cnt             = 0;

    // CONSTRUCTORES
    /**
     * Constructor para crear objetos de la clase Factura
     */
    public Factura() {
        this(1); // asumo que el nro de la factura es 1
    }
    public Factura(int n) {
        this(n,new Date());
    }
    public Factura(int n,Date f) {
        this(n,f,new Cliente());
    }
    public Factura(int n,Date f,Cliente c) {
        setNro(n);
        setFecha(f);
        setCliente(c);
    }
    
    // GETTERS
    public int getNro()             { return nro;   }
    public Date getFecha()          { return fecha; }
    // fecha en formato dd/mm/yyyy
    public String getFechaStr()     { 
        SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
        String strFecha = sdf.format(getFecha());
        return strFecha; 
    }
    public Cliente getCliente()     { return cliente; }
    public int getCount()           { return cnt; }
    public boolean isConfirmada()   { return confirmada; }
    public boolean isPagada()       { return pagada; }
    public double getMonto()        { 
        calcular();
        return monto; 
    }
    
    // SETTERS
    public void setNro(int n)           { nro = n; }
    public void setFecha(Date d)        { 
        if ( isConfirmada() ) return; // no hago nada si la factura esta confirmada
        fecha = d; 
    }
    public void setFecha(String d)      { 
        if ( isConfirmada() ) return; // no hago nada si la factura esta confirmada
        SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
        try {
            Date f = sdf.parse(d);
            setFecha(f);
        } catch(ParseException pe) {
            System.err.println("Clase Factura metodo setFecha(): Error, ["+d+"] no es una fecha en formato dd/MM/yyyy");
        }
    }
    public void setCliente(Cliente c)   { 
        if ( isConfirmada() ) return; // no hago nada si la factura esta confirmada
        cliente = c; 
    }
    
    // CUSTOM
    public void confirmar()                 {
        if ( isConfirmada() ) return; // no hago nada si la factura esta confirmada
        if ( getCliente() == null ) return; // no hay cliente asociado a la factura
        if ( getCount() <= 0 ) return; // no hay detalle de factura
        calcular(); // recalcula monto de la factura
        // sumo al saldo deudor del cliente
        double saldo = getCliente().getSaldo();
        saldo+=getMonto();
        getCliente().setSaldo(saldo);
        // descuento stock de los productos
        for(Detalle d : detalle) {
            if ( d != null ) d.getProducto().addStock(d.getCantidad()*(-1));
        }
        confirmada = true;
    } 
    public void pagar()                     {
        if ( isPagada() ) return; // no puedo pagar mas de una vez a la factura
        if ( isConfirmada() ) {
            double saldo = getCliente().getSaldo();
            saldo-=getMonto();
            getCliente().setSaldo(saldo);
        }
    } 
    private void calcular()                 {
        // recorro todos los objetos Detalle y sumarizo
        monto = 0.0;
        for(Detalle d : detalle) {
            if ( d != null ) monto+=d.getSubtotal();
        }
    }
    public void addDetalle(Detalle d)       {
        if ( !isConfirmada() ) {
            // puedo agregar detalle
            if ( getCount() < detalle.length ) {
                // hay lugar para el nuevo detalle
                for(int i=0;i < detalle.length; i++) {
                    if ( detalle[i] == null ) {
                        // encontre un lugar vacio para poner la referencia a este objeto Detalle
                        detalle[i] = d;
                        cnt++;
                        break;
                    }
                }
            } else {
                System.err.println("addDetalle(): Error, Factura llena!");
            }
        } else {
            System.err.println("addDetalle(): Error, factura confirmada!");
        }
    } 
    public void addDetalle(Detalle d,int p) {
        if ( p >= 0 && p < detalle.length ) {
            if ( !isConfirmada() ) {
                // puedo agregar detalle
                if ( getCount() < detalle.length ) {
                    // hay lugar para el nuevo detalle
                    if ( detalle[p] == null ) {
                        detalle[p] = d;
                        cnt++;
                    } else { // esta posicion ya estaba ocupada por otro objeto Detalle
                        detalle[p] = d;
                    }
                } else {
                    System.err.println("addDetalle(): Error, Factura llena!");
                }
            } else {
                System.err.println("addDetalle(): Error, factura confirmada!");
            }
        } else {
            System.err.println("addDetalle(): Error, posicion incorrecta!");
        }
    } 
    public void delDetalle(Detalle d)       {
        if (!isConfirmada() && getCount() > 0) {
            for(int i=0;i < detalle.length; i++) {
                Detalle dd = detalle[i];
                if ( dd != null ) {
                    if ( dd.getProducto().getIdp() == d.getProducto().getIdp() &&
                         dd.getCantidad() == d.getCantidad() &&
                         dd.getPrecio() == d.getPrecio() ) {
                         detalle[i] = null;
                         cnt--;
                         break;
                    }
                }
            }
        }
    } 
    public void delDetalle(int p)           {
        if ( p >= 0 && p < detalle.length ) {
            if ( !isConfirmada() && getCount() > 0 ) {
                // puedo borrar detalle
                if ( detalle[p] != null ) {
                    detalle[p] = null;
                    cnt--;
                }
            } else {
                System.err.println("delDetalle(): Error, factura confirmada!");
            }
        } else {
            System.err.println("delDetalle(): Error, posicion incorrecta!");
        }
    }
}
